function [Vs Ts] = fvst(s,gamma,sigma,order,ft,bt)
% Computes the STFT Vs, the SST Ts and the second-order reassigned STFT Rs of the signal s.
% The only difference with sst is the definition of \omega in the
% reassignment step in the SST. If order==1, standard SST. If order==2,
% second-order SST which perfectly localizes linear chirps.


% checking length of signal
n = length(s);
nv = log2(n);
if mod(nv,1)~=0
    warning('The signal is not a power of two, truncation to the next power');
    s = s(1:2^mod(nv,1));
end
n = length(s);
s = s(:);

% Optional parameters
if nargin<6
    if nargin<4
        order=1;
    end
   ft = 1:n/2;
   bt = 1:n;
end

% Padding
sleft = flipud(s(2:n/2+1));
sright = flipud(s(end-n/2:end-1));
x = [sleft; s; sright];
clear xleft xright;

%% STFT and operator omega
nb = length(bt);
neta = length(ft);
df = ft(2)-ft(1);
db = bt(2)-bt(1);

t = linspace(-0.5,0.5,n);t=t';

% Gaussian
g = exp(-pi/sigma^2*t.^2);
gp = -2*pi/sigma^2*t .* g; % g'

Vs = zeros(neta,nb);
Ts = zeros(neta,nb);
omega = zeros(neta,nb);


for b=1:nb
    % STFT
    tmp = fft(x(bt(b):bt(b)+n-1).*g)/n/sigma;
    Vs(:,b) = tmp(ft);
end

if order==1    
    for b=1:nb
        % omega
        tmp = fft(x(bt(b):bt(b)+n-1).*gp)/n/sigma;
        omega(:,b) = (ft-1)'-real(tmp(ft)/2/1i/pi./Vs(:,b));
    end
    
elseif order==2
    for b=1:nb
        tmp = fft(x(bt(b):bt(b)+n-1).*g)/n/sigma;
        vg = tmp(ft);
        tmp = fft(x(bt(b):bt(b)+n-1).*gp)/n/sigma;
        vgp = tmp(ft);
        tmp = fft(x(bt(b):bt(b)+n-1).*g.*t)/n/sigma;
        vxg = tmp(ft);
        
        tmp = -real(vgp/2/1i/pi./vg);
        domeg = tmp(ft);
        
        phip = (real(vxg./vg))./real(vgp/2/1i/pi./vg)/sigma^4;
        phip(abs(domeg)<0.0001)=0;
        
        omega(:,b) = (ft-1)' + (1+phip.^2*sigma^4) .* domeg;
    end
    
end

    
%% FVST : vertical reassignment
for b=1:nb
    % Reassignment step
    for eta=1:neta
        if abs(Vs(eta,b))>gamma
            k = 1+round((omega(eta,b)-ft(1)+1)/df);
            if k>=1 && k<=neta
                Ts(k,b) = Ts(k,b) + Vs(eta,b)*exp(1i*pi*(ft(eta)-1));
            end
        end
    end
end
Ts = df*sigma*Ts;

end