function spm_fig2()
%SPM_FIG2: plots Figure 3 of paper "RECENT ADVANCES IN TIME-FREQUENCY REASSIGNMENT AND
%   SYNCHROSQUEEZING", by Meignen et al.


N = 4*1024;
T = 1;
t = (0:N-1)/N;
un = ones(size(t));

%% Multicomponent signal

a = exp(-8*(t-T/2*un).^2);
s1 = a.*sin(2*pi*(250*t+100*t.^3));
s2 = a.*sin(2*pi*(130*t+100*t.^2));
s3 = a.*sin(2*pi*(90*t+0.2*cos(3*pi*t)));
s = s1+s2+s3;

fs = N;
frecv = 1:512;
timev = 1:N;
s = s/norm(s)*sqrt(N);

% Parameters
sigma=0.025;
lambda = 0.02;
gamma = 0.05;

[Vs Ts Rs] = sst(s,gamma,sigma,frecv,timev);

% zoom coordinates
xn=0.2;xN=0.8;
yn=320;yN=450;

%% Display

% SST
ha = subplot(1,3,1);
tfplot(Ts,t(timev),(frecv-1)*fs/N,ha);set(ha,'xlim',[xn xN]);
set(ha,'xtick',[],'ytick',[]);xlabel('');ylabel('');

% Ridge extraction
ha = subplot(1,3,2);
[C, Es] = brevridge_mult(Ts, frecv, 3, lambda, 25);
mycol = {'blue','black','red'};
hold(ha);
for k=1:3
    plot(t,frecv(C(k,:)),'color',mycol{k},'Parent',ha,'LineWidth',2);
end
set(ha,'xtick',[],'ytick',[]);xlabel('');ylabel('');
set(ha,'xlim',[xn xN]);

% IMF plot
srec1 = zeros(size(s));
srec2 = zeros(size(s));
srec3 = zeros(size(s));
Delta = 12;
for b=1:length(timev)
	srec1(b) = sum(Ts(max(1,C(1,b)-Delta):min(length(frecv),C(1,b)+Delta),b));
    srec2(b) = sum(Ts(max(1,C(2,b)-Delta):min(length(frecv),C(2,b)+Delta),b));
    srec3(b) = sum(Ts(max(1,C(3,b)-Delta):min(length(frecv),C(3,b)+Delta),b));
end
srec1 = 2*real(srec1);
srec2 = 2*real(srec2);
srec3 = 2*real(srec3);
ha = subplot(1,3,3);
plot(t,srec3,'r',t,srec2,'k',t,srec1,'b');
legend('Mode 1','Mode 2','Mode 3');
set(ha,'xlim',[0.5 0.54]);
xlabel('t');





end

