function spm_order2()
% plots Figure 6 (A) of paper "RECENT ADVANCES IN TIME-FREQUENCY REASSIGNMENT AND
%   SYNCHROSQUEEZING", by Meignen et al.

N = 1024;
T = 1;
t = (0:N-1)/N;
f = (0:N/2-1)/2/pi;
un = ones(size(t));

% Multicomponent signal
a = exp(-8*(t-T/2*un).^2);
s1 = a.*sin(2*pi*(250*t+100*t.^3));
so = s1;
so = so(:);

% normalize
so = so/norm(so)*sqrt(length(so));

% Parameters
N = length(so);
fs = N;
t = 1/fs*(0:N-1);
frecv = 0:N/2-1;


% Parameters
lambda = 0.02;
clwin = 50;
gamma = 0.00001;
index = N/8:7*N/8;
sigma = 0.03;
nmodes = 1;
del = 0;

% SST
[Vs Ts1 Rs] = sst(so,gamma,sigma); % order 1
[Vs Ts2] = fvst(so,gamma,sigma,2); % order 2

% Display TF representation
%tfplot(Ts1,t,f);return;

% Extraction
[r1, phi] = brevridge_mult(Ts1,frecv, nmodes, lambda, clwin);
[r2, phi] = brevridge_mult(Ts2,frecv, nmodes, lambda, clwin);

% Reconstruction
srec1 = zeros(size(so));
for kl=1:N
	for jl=1:nmodes
    	srec1(kl) = srec1(kl) + sum(Ts1(max(1,r1(jl,kl)-del):min(r1(jl,kl)+del,N/2),kl));
    end
end
srec1 = 2*real(srec1);
err1 = -20*log(norm(so(index)-srec1(index))/norm(so(index)));

srec2 = zeros(size(so));
for kl=1:N
	for jl=1:nmodes
        srec2(kl) = srec2(kl) + sum(Ts2(max(1,r2(jl,kl)-del):min(r2(jl,kl)+del,N/2),kl));
    end
end
srec2 = 2*real(srec2);
err2 = -20*log(norm(so(index)-srec2(index))/norm(so(index)));


% Display
figure();

ha = subplot(2,2,1);
tfplot(Ts1,t,f,ha);
set(ha,'xlim',[0.7 0.8],'ylim',[61 72]);
title('(A)');

ha = subplot(2,2,3);
tfplot(Ts2,t,f,ha);
set(ha,'xlim',[0.7 0.8],'ylim',[61 72]);
title('(B)');

ha = subplot(2,2,2);
plot(t,srec1,'b',t,so,'r--');
set(ha,'xlim',[0.74 0.76],'ylim',[-2 2]);
xlabel('t');
title('(C)');

ha = subplot(2,2,4);
plot(t,srec2,'b',t,so,'r--');
set(ha,'xlim',[0.74 0.76],'ylim',[-2 2]);
xlabel('t');
title('(D)');

explot();
set(findall(0,'type','Line'),'LineWidth',2);
set(findall(0,'type','axes'),'ytick',[]);

disp(err1);
disp(err2);


